#!/usr/bin/perl
#Attribute.pm
#The Attribute class represents an attribute title, a list of values, and an iterator.
#Written by Paul Stothard.

package ARFF::Attribute;

use strict;
use warnings;

sub new {
    my $object = shift;
    my $class  = ref($object) || $object;
    my $self   = {
        title    => undef,
        values   => [],
        iterator => 0
    };
    bless( $self, $class );
    return $self;
}

sub setTitle {
    my $self = shift;
    my $title;
    $title = shift if (@_);
    $self->{title} = $title;
}

sub getTitle {
    my $self = shift;
    return $self->{title};
}

sub addValue {
    my $self = shift;
    my $value;
    if (@_) {
        $value = shift;
    }
    else {
        return 0;
    }
    my $arrayReference = $self->{values};
    push( @$arrayReference, $value );
}

sub resetIterator {
    my $self = shift;
    $self->{iterator} = 0;
}

sub hasNext {
    my $self           = shift;
    my $arrayReference = $self->{values};
    if ( $self->{iterator} < scalar(@$arrayReference) ) {
        return 1;
    }
    else {
        return 0;
    }
}

sub getNextValue {
    my $self           = shift;
    my $arrayReference = $self->{values};
    my $iterator       = $self->{iterator};
    if ( $iterator < $self->getNumberOfValues() ) {
        $self->advanceIterator();
        return $arrayReference->[$iterator];
    }
    else {
        return 0;
    }
}

sub getNumberOfValues {
    my $self           = shift;
    my $arrayReference = $self->{values};
    return scalar(@$arrayReference);
}

sub write {
    my $self = shift;
    print $self->toString;
}

sub toString {
    my $self           = shift;
    my $arrayReference = $self->{values};

    my %hash = ();
    foreach (@$arrayReference) {
        $hash{$_} = 1;
    }
    my @keys = keys(%hash);
    @keys = sort(@keys);
    my $joined = join( ", ", @keys );

    return "\@attribute " . $self->{title} . " { " . $joined . " }\n";
}

return 1;
