#!/usr/bin/perl
#Relation.pm
#The Relation class represents a relation title, a list of attributes, and a list of data.
#Written by Paul Stothard.

package ARFF::Relation;

use strict;
use warnings;

sub new {
    my $object = shift;
    my $class  = ref($object) || $object;
    my $self   = {
        title      => undef,
        attributes => [],
        data       => []
    };
    bless( $self, $class );
    return $self;
}

sub setTitle {
    my $self = shift;
    my $title;
    $title = shift if (@_);
    $self->{title} = $title;
}

sub getTitle {
    my $self = shift;
    return $self->{title};
}

sub addAttribute {
    my $self = shift;
    my $value;
    if (@_) {
        $value = shift;
    }
    else {
        return 0;
    }
    my $arrayReference = $self->{attributes};
    push( @$arrayReference, $value );
}

sub addData {
    my $self = shift;
    my $value;
    if (@_) {
        $value = shift;
    }
    else {
        return 0;
    }
    my $arrayReference = $self->{data};
    push( @$arrayReference, $value );
}

sub write {
    my $self = shift;
    my $now  = gmtime;

    #write the title
    print "\% Title: multiple sequence alignment relation\n";
    print "\% Creator: align_to_arff.pl on $now (GM time)\n";
    print "\@relation " . $self->{title} . "\n\n";

    #write the attributes
    my $arrayReference = $self->{attributes};
    foreach (@$arrayReference) {
        $_->write();
    }

    $arrayReference = $self->{data};

    print "\n";
    print "\@data\n";
    print "\% \n";
    print "\% " . scalar(@$arrayReference) . " instances\n";
    print "\% \n";

    #write the data
    foreach (@$arrayReference) {
        $_->write();
    }

}

sub toString {
    my $self = shift;
    my $now  = gmtime;
    my $output;

    #write the title
    $output = "\% Title: multiple sequence alignment relation\n";
    $output = $output . "\% Creator: align_to_arff.pl on $now (GM time)\n";
    $output = $output . "\@relation " . $self->{title} . "\n\n";

    #write the attributes
    my $arrayReference = $self->{attributes};
    foreach (@$arrayReference) {
        $output = $output . $_->toString();
    }

    $arrayReference = $self->{data};

    $output = $output . "\n";
    $output = $output . "\@data\n";
    $output = $output . "\% \n";
    $output = $output . "\% " . scalar(@$arrayReference) . " instances\n";
    $output = $output . "\% \n";

    #write the data
    foreach (@$arrayReference) {
        $output = $output . $_->toString();
    }
    return $output;
}

return 1;
