#!/usr/bin/perl
#Sequence.pm
#The Sequence class represents a sequence title, sequence proper, and provides methods for obtaining specific residues.
#Written by Paul Stothard.

package ARFF::Sequence;

use strict;
use warnings;

sub new {
    my $object = shift;
    my $class  = ref($object) || $object;
    my $self   = {
        title    => undef,
        class    => undef,
        residues => []
    };
    bless( $self, $class );
    return $self;
}

sub setTitle {
    my $self = shift;
    my $title;
    $title = shift if (@_);
    $self->{title} = $title;
}

sub setClass {
    my $self = shift;
    my $class;
    $class = shift if (@_);
    $self->{class} = $class;
}

sub setSequence {
    my $self = shift;
    my $sequence;
    $sequence = shift if (@_);
    my @arrayOfBases = split( /\B|\b/, $sequence );
    $self->{residues} = \@arrayOfBases;
}

sub getTitle {
    my $self = shift;
    return $self->{title};
}

sub getClass {
    my $self = shift;
    return $self->{class};
}

sub getLength {
    my $self           = shift;
    my $arrayReference = $self->{residues};
    return scalar(@$arrayReference);
}

sub getResidue {
    my $self = shift;
    my $residue;
    my $arrayReference = $self->{residues};
    if (@_) {
        $residue = shift;
    }
    else {
        die("Sequence class get_residue method requires base parameter");
    }
    if ( $residue > scalar(@$arrayReference) ) {
        die("Residue value $residue cannot be longer than sequence");
    }
    if ( $residue < 1 ) {
        die("Residue value must be greater than 1");
    }
    return $arrayReference->[ $residue - 1 ];
}

return 1;
